/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSStyleSheet.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMImplementation.h"
#include "DOMWindow.h"
#include "Document.h"
#include "DocumentType.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLDocument.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSStyleSheet.h"
#include "gdom/GdomDOMImplementation.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomDocument.h"
#include "gdom/GdomDocumentType.h"
#include "gdom/GdomHTMLDocument.h"
#include "gdom/PrivateGdomCSSStyleSheet.h"
#include "gdom/PrivateGdomDOMImplementation.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDocument.h"
#include "gdom/PrivateGdomDocumentType.h"
#include "gdom/PrivateGdomHTMLDocument.h"
#include "gdom/PrivateIntGdomCSSStyleSheet.h"
#include "gdom/PrivateIntGdomDOMImplementation.h"
#include "gdom/PrivateIntGdomDocument.h"
#include "gdom/PrivateIntGdomDocumentType.h"
#include "gdom/PrivateIntGdomHTMLDocument.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::DOMImplementation* obj)
{
    return WebCore::cacheDOMObject<WebCore::DOMImplementation, GDOMDOMImplementation>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMDOMImplementation::GDOMDOMImplementation(PassRefPtr<WebCore::DOMImplementation> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMDOMImplementation::~GDOMDOMImplementation()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

gboolean
gdom_dom_implementation_has_feature (GdomDOMImplementation *thiz, gchar * feature, gchar * version)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMImplementation * item = (gdom_dom_implementation_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (feature, 0);
    g_return_val_if_fail (version, 0);
    WebCore::String _g_feature = WebCore::String::fromUTF8(feature);
    WebCore::String _g_version = WebCore::String::fromUTF8(version);
    gboolean res = item->hasFeature(_g_feature, _g_version);
    return res;

}

GdomDocumentType *
gdom_dom_implementation_create_document_type (GdomDOMImplementation *thiz, gchar * qualified_name, gchar * public_id, gchar * system_id)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMImplementation * item = (gdom_dom_implementation_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (qualified_name, 0);
    g_return_val_if_fail (public_id, 0);
    g_return_val_if_fail (system_id, 0);
    WebCore::String _g_qualified_name = WebCore::String::fromUTF8(qualified_name);
    WebCore::String _g_public_id = WebCore::String::fromUTF8(public_id);
    WebCore::String _g_system_id = WebCore::String::fromUTF8(system_id);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::DocumentType> g_res = WTF::getPtr(item->createDocumentType(_g_qualified_name, _g_public_id, _g_system_id, ec));
    GdomDocumentType * res = static_cast<GdomDocumentType *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomDocument *
gdom_dom_implementation_create_document (GdomDOMImplementation *thiz, gchar * namespace_uri, gchar * qualified_name, GdomDocumentType * doctype)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMImplementation * item = (gdom_dom_implementation_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (qualified_name, 0);
    g_return_val_if_fail (doctype, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_qualified_name = WebCore::String::fromUTF8(qualified_name);
    WebCore::DocumentType * _g_doctype = (gdom_document_type_get_private(doctype));
    g_return_val_if_fail (_g_doctype, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Document> g_res = WTF::getPtr(item->createDocument(_g_namespace_uri, _g_qualified_name, _g_doctype, ec));
    GdomDocument * res = static_cast<GdomDocument *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomCSSStyleSheet *
gdom_dom_implementation_create_css_style_sheet (GdomDOMImplementation *thiz, gchar * title, gchar * media)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMImplementation * item = (gdom_dom_implementation_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (title, 0);
    g_return_val_if_fail (media, 0);
    WebCore::String _g_title = WebCore::String::fromUTF8(title);
    WebCore::String _g_media = WebCore::String::fromUTF8(media);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::CSSStyleSheet> g_res = WTF::getPtr(item->createCSSStyleSheet(_g_title, _g_media, ec));
    GdomCSSStyleSheet * res = static_cast<GdomCSSStyleSheet *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomHTMLDocument *
gdom_dom_implementation_create_html_document (GdomDOMImplementation *thiz, gchar * title)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMImplementation * item = (gdom_dom_implementation_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (title, 0);
    WebCore::String _g_title = WebCore::String::fromUTF8(title);
    PassRefPtr<WebCore::HTMLDocument> g_res = WTF::getPtr(item->createHTMLDocument(_g_title));
    GdomHTMLDocument * res = static_cast<GdomHTMLDocument *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "DOMImplementation.h"
#include <gdom/GdomDOMImplementation.h>
#include <gdom/PrivateGdomDOMImplementation.h>
#include <gdom/PrivateIntGdomDOMImplementation.h>
#include <gdom/GdomDOMImplementationprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDOMImplementation, gdom_dom_implementation, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomDOMImplementationPrivate {
    class WebCore::DOMImplementation *item;
};

#define GDOM_DOM_IMPLEMENTATION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DOM_IMPLEMENTATION, GdomDOMImplementationPrivate))
*/
G_END_DECLS
WebCore::DOMImplementation *
gdom_dom_implementation_get_private(GdomDOMImplementation *request)
{
    GDOMDOMImplementation *res;
    res = static_cast<GDOMDOMImplementation*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::DOMImplementation*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
};


static void gdom_dom_implementation_finalize(GObject* object)
{
    /*GdomDOMImplementation* request = GDOM_DOM_IMPLEMENTATION(object);*/
    /*GdomDOMImplementationPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_dom_implementation_parent_class)->finalize(object);
}

static void gdom_dom_implementation__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMImplementation *self = GDOM_DOM_IMPLEMENTATION(object);
    WebCore::DOMImplementation * gself = (gdom_dom_implementation_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_implementation__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMImplementation *self = GDOM_DOM_IMPLEMENTATION(object);
    WebCore::DOMImplementation * gself = (gdom_dom_implementation_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_implementation_class_init(GdomDOMImplementationClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_dom_implementation_finalize;
    g_klass->set_property = gdom_dom_implementation__set_property_fn;
    g_klass->get_property = gdom_dom_implementation__get_property_fn;





    /*g_type_class_remove_private(requestClass, sizeof(GdomDOMImplementationPrivate));*/
}

static void gdom_dom_implementation_init(GdomDOMImplementation* request)
{
    /*GdomDOMImplementationPrivate* priv = GDOM_DOM_IMPLEMENTATION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

